﻿using CsvHelper;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Meecat.JoinData
{
    class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine($"{DateTime.Now} reading pdata.csv");
            var pipes = new List<pipe>();
            using (var reader = new CsvReader(new StreamReader("pdata.csv")))
            {
                while (reader.Read())
                {
                    //Part ID, Install Year,Diameter Size, Length, Material
                    var pipe = new pipe();
                    pipe.part_id = reader.GetField("Part ID");
                    pipe.part_year = reader.GetField<int>("Install Year");
                    pipe.diameter = reader.GetField<int>("Diameter Size");
                    pipe.length = reader.GetField<double>("Length");
                    pipe.material = reader.GetField("Material");

                    pipes.Add(pipe);
                }
            }

            Console.WriteLine($"{DateTime.Now} reading pfail.csv");
            var failpipes = new List<failpipe>();
            using (var reader = new CsvReader(new StreamReader("pfail.csv")))
            {
                while (reader.Read())
                {
                    //Part ID, Failure_Year
                    var pipe = new failpipe();
                    pipe.part_id = reader.GetField("Part ID");
                    pipe.failure_year = reader.GetField<int>("Failure_Year");

                    failpipes.Add(pipe);
                }
            }

            var pipeDict = pipes.ToDictionary(c => c.part_id, v => v);

            Console.WriteLine($"{DateTime.Now} combining...");
            foreach (var pipe in failpipes)
            {
                if (pipeDict.ContainsKey(pipe.part_id))
                {
                    var p = pipeDict[pipe.part_id];
                    pipe.part_year = p.part_year;
                    pipe.diameter = p.diameter;
                    pipe.length = p.length;
                    pipe.material = p.material;
                }
                else
                {
                    Console.WriteLine(pipe.part_id);
                }
            }

            Console.WriteLine($"{DateTime.Now} ordering...");

            var orderpipes = (from p in failpipes
                             where p.part_year>0
                             orderby p.part_id, p.failure_year, p.part_year
                             select p).ToList();
            
            Console.WriteLine($"{DateTime.Now} saving...");
            //var json = JsonConvert.SerializeObject(orderpipes);
            //File.WriteAllText("p_fail_events.json", json);
            //Console.WriteLine(json);
            using(var writer = new CsvWriter(new StreamWriter(new FileStream("p_fail_events.csv", FileMode.OpenOrCreate))))
            {
                writer.WriteRecords(orderpipes);
            }
            Console.WriteLine($"{DateTime.Now} done!");
            Console.Read();
        }
    }

    class pipe
    {
        public string part_id { get; set; }
        public int part_year { get; set; }
        public int diameter { get; set; }
        public double length { get; set; }
        public string material { get; set; }
    }

    class failpipe 
    {
        public string part_id { get; set; }
        public int part_year { get; set; }
        public int diameter { get; set; }
        public double length { get; set; }
        public string material { get; set; }
        public int failure_year { get; set; }
    }
}
